import { z } from "zod";
import { useForm } from "react-hook-form";
import { zodResolver } from "@hookform/resolvers/zod";
import { Button } from "@/components/ui/button";
import {
	Form,
	FormControl,
	FormField,
	FormItem,
	FormLabel,
	FormMessage
} from "@/components/ui/form";
import { PasswordInput } from "@/components/password-input";
import { toast } from "@/hooks/useToast";

const passwordChangeSchema = z
	.object({
		oldPassword: z.string().min(1, {
			message: "Please enter your existing password"
		}),
		newPassword: z
			.string()
			.min(1, { message: "Please enter your new password" })
			.min(8, { message: "Password must be at least 8 characters long" })
			.max(128, { message: "Password must be at most 128 characters long" })
			.refine((password) => /[A-Z]/.test(password), {
				message: "Password must contain at least one uppercase letter"
			})
			.refine((password) => /[a-z]/.test(password), {
				message: "Password must contain at least one lowercase letter"
			})
			.refine((password) => /\d/.test(password), {
				message: "Password must contain at least one number"
			})
			.refine((password) => /[@$!%*?&]/.test(password), {
				message:
					"Password must contain at least one special character (@, $, !, %, *, ?, &)"
			}),
		confirmNewPassword: z.string().min(1, {
			message: "Please confirm your new password"
		})
	})
	.refine((data) => data.newPassword === data.confirmNewPassword, {
		message: "Passwords don't match.",
		path: ["confirmNewPassword"]
	});

type PasswordChangeFormValues = z.infer<typeof passwordChangeSchema>;

export default function PasswordChangeForm() {
	const form = useForm<PasswordChangeFormValues>({
		resolver: zodResolver(passwordChangeSchema),
		defaultValues: {
			oldPassword: "",
			newPassword: "",
			confirmNewPassword: ""
		}
	});

	function onSubmit(data: PasswordChangeFormValues) {
		toast({
			title: "Password Changed (MOCK)",
			description: "Your password has been updated."
		});
	}

	return (
		<Form {...form}>
			<form onSubmit={form.handleSubmit(onSubmit)} className="space-y-8">
				<FormField
					control={form.control}
					name="oldPassword"
					render={({ field }) => (
						<FormItem>
							<FormLabel>Existing Password</FormLabel>
							<FormControl>
								<PasswordInput
									placeholder="Enter current password"
									{...field}
								/>
							</FormControl>
							<FormMessage />
						</FormItem>
					)}
				/>
				<FormField
					control={form.control}
					name="newPassword"
					render={({ field }) => (
						<FormItem>
							<FormLabel>New Password</FormLabel>
							<FormControl>
								<PasswordInput placeholder="Enter new password" {...field} />
							</FormControl>
							<FormMessage />
						</FormItem>
					)}
				/>
				<FormField
					control={form.control}
					name="confirmNewPassword"
					render={({ field }) => (
						<FormItem>
							<FormLabel>Confirm New Password</FormLabel>
							<FormControl>
								<PasswordInput placeholder="Confirm new password" {...field} />
							</FormControl>
							<FormMessage />
						</FormItem>
					)}
				/>
				<Button type="submit">Change Password</Button>
			</form>
		</Form>
	);
}
